from main import app
from auth import auth

ADMIN_ONLY_SVC = "admin"
USER_AND_ADMIN_SVC = "admin"
USER_ONLY_SVC = "user"
OTHER_ONLY_SVC = "other"


def override_require_authorized_user(permissions, data_partition_id="osdu"):
    async def authf():
        auth_data = auth.Auth(
            access_token="Bearer FakeToken", data_partition_id=data_partition_id
        )
        if permissions is ADMIN_ONLY_SVC:
            auth_data.groups = [
                {
                    "name": "service.policy.admin",
                    "description": "Policy admin group",
                    "email": "service.policy.admin@osdu.group",
                }
            ]
        elif permissions is USER_AND_ADMIN_SVC:
            auth_data.groups = [
                {
                    "name": "service.policy.user",
                    "description": "Policy user group",
                    "email": "service.policy.user@osdu.group",
                },
                {
                    "name": "service.policy.admin",
                    "description": "Policy admin group",
                    "email": "service.policy.admin@osdu.group",
                },
            ]
        elif permissions is USER_ONLY_SVC:
            auth_data.groups = [
                {
                    "name": "service.policy.user",
                    "description": "Policy user group",
                    "email": "service.policy.user@osdu.group",
                }
            ]
        elif permissions is OTHER_ONLY_SVC:
            auth_data.groups = [
                {
                    "name": "service.other.user",
                    "description": "other user group",
                    "email": "service.other.user@osdu.group",
                }
            ]
        else:
            auth_data.groups = [{}]
        auth_data.authenticated = True
        return auth_data

    return authf


def set_authorize_session(permissions=None, data_partition_id="osdu"):
    def decorator_func(func):
        def wrapper_func(*args, **kwargs):
            print(f"adding dependency {permissions}")
            app.dependency_overrides[
                auth.require_authorized_user
            ] = override_require_authorized_user(
                permissions=permissions, data_partition_id=data_partition_id
            )
            if permissions is ADMIN_ONLY_SVC:
                app.dependency_overrides[
                    auth.require_authorized_admin
                ] = override_require_authorized_user(
                    permissions=permissions, data_partition_id=data_partition_id
                )
            try:
                returned_value = func(*args, **kwargs)
                print(f"removing dependency {permissions}")
                app.dependency_overrides = {}
                return returned_value
            except Exception as e:
                print(f"removing dependency {permissions}")
                app.dependency_overrides = {}
                raise e

        return wrapper_func

    return decorator_func
